/*
   Copyright (C) 2016 Ignacio Vina (@igvina)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#pragma once

#define BALL_INVISIBLE        0
#define BALL_SMALL            1
#define BALL_MEDIUM           2
#define BALL_BIG              3

#define GIFT_DOUBLE_RAY       B00000100
#define GIFT_NEGATIVE         B00001000
#define GIFT_MIRROR           B00010000
#define GIFT_SHIELD           B00100000
//Only one can be at the same time
#define GIFT_SLOW             B01000000
#define GIFT_FAST             B10000000


// Not direct on level
#define GIFT_LIFE             B01100000
#define GIFT_DEATH            B11000000



//Only on level
#define NO_LUCKY_GIFTS        B00000010

//Default move right & down
#define INVERTED_MOTION       B10000000


#define BACKGROUND_MIRROR     B10000000

// Noise reduces fps from 60 to 50, not fast enough
#define BACKGROUND_NOISE      B01000000

#define ROTATING_BALLS        B10000000

// 1 - int (noise = rand() % NOISE_LEVEL == 0)
#define NOISE_LEVEL    10

#define BACKGROUND_MOUNTAIN           (0 << 4)
#define BACKGROUND_CITY               (1 << 4)
#define BACKGROUND_MOON               (2 << 4)
#define BACKGROUND_EMPTY_MOON         (3 << 4)


//NUMBER OF INITIAL BALLS (MAX 15) (4bits) | BACKGROUND | BACKGROUND (OPT) ,
//INITIAL LEVEL ITEMS | NO_LUCKY_GIFTS (OPT)
//INITIAL PLAYER X POSITION | ROTATING BALLS,
//BALL TYPE 1, X POS 1 | MOTION (OPT), Y POS 1 | MOTION (OPT),
//...
//BALL TYPE N, ...


//
// LEARN TO PLAY (LTP) LEVELS
//

const unsigned char PROGMEM LTP_A1[] = {
  1 | BACKGROUND_MOUNTAIN,
  NO_LUCKY_GIFTS,
  32,
  BALL_MEDIUM | GIFT_LIFE, 38, 14,
};

const unsigned char PROGMEM LTP_A2[] = {
  1 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  28,
  BALL_BIG | GIFT_DOUBLE_RAY, 106 | INVERTED_MOTION, 44,
};

const unsigned char PROGMEM LTP_A3[] = {
  2 | BACKGROUND_MOUNTAIN,
  NO_LUCKY_GIFTS,
  16,
  BALL_MEDIUM, 96 | INVERTED_MOTION, 20,
  BALL_SMALL | GIFT_SHIELD , 64 | INVERTED_MOTION, 40,
};

const unsigned char PROGMEM LTP_A4[] = {
  2 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  64,
  BALL_MEDIUM | GIFT_SLOW, 30, 14,
  BALL_MEDIUM | GIFT_SLOW, 98 | INVERTED_MOTION, 14,
};

const unsigned char PROGMEM LTP_A5[] = {
  4 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  58,
  BALL_SMALL | GIFT_DEATH, 20, 15,
  BALL_SMALL | GIFT_LIFE, 48, 25,
  BALL_SMALL, 80, 25,
  BALL_SMALL, 108, 15,
};


const unsigned char PROGMEM LTP_B1[] = {
  3 | BACKGROUND_CITY,
  NO_LUCKY_GIFTS,
  64,
  BALL_MEDIUM, 30, 14,
  BALL_SMALL | GIFT_NEGATIVE, 64, 30,
  BALL_MEDIUM, 98, 14,
};

const unsigned char PROGMEM LTP_B2[] = {
  2 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS | GIFT_DOUBLE_RAY,
  24,
  BALL_MEDIUM | GIFT_LIFE, 64, 32,
  BALL_BIG | GIFT_SHIELD, 104, 24,
};

const unsigned char PROGMEM LTP_B3[] = {
  2 | BACKGROUND_CITY,
  NO_LUCKY_GIFTS,
  100,
  BALL_SMALL | GIFT_FAST, 85, 18,
  BALL_MEDIUM, 13, 13,
};

const unsigned char PROGMEM LTP_B4[] = {
  3 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  64,
  BALL_SMALL, 12 | INVERTED_MOTION, 52,
  BALL_SMALL | GIFT_MIRROR, 54, 12,
  BALL_SMALL, 116, 52,
};

const unsigned char PROGMEM LTP_B5[] = {
  1 | BACKGROUND_CITY,
  NO_LUCKY_GIFTS | GIFT_FAST | GIFT_DOUBLE_RAY,
  64,
  BALL_BIG | GIFT_SHIELD, 28, 39,
};


const unsigned char PROGMEM LTP_C1[] = {
  6 | BACKGROUND_MOON,
  NO_LUCKY_GIFTS,
  64,
  BALL_SMALL | GIFT_DOUBLE_RAY, 32 | INVERTED_MOTION, 26 | INVERTED_MOTION,
  BALL_SMALL, 44 | INVERTED_MOTION, 33 | INVERTED_MOTION,
  BALL_SMALL, 56 | INVERTED_MOTION, 40 | INVERTED_MOTION,
  BALL_SMALL, 72, 40 | INVERTED_MOTION,
  BALL_SMALL, 84, 33 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 96, 26 | INVERTED_MOTION,
};

const unsigned char PROGMEM LTP_C2[] = {
  2 | BACKGROUND_EMPTY_MOON | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS | GIFT_MIRROR,
  64,
  BALL_MEDIUM | GIFT_MIRROR, 20, 42,
  BALL_MEDIUM | GIFT_MIRROR, 108 | INVERTED_MOTION, 42,
};

const unsigned char PROGMEM LTP_C3[] = {
  4 | BACKGROUND_MOON | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS | GIFT_NEGATIVE,
  64,
  BALL_SMALL | GIFT_NEGATIVE, 16 | INVERTED_MOTION, 26 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_NEGATIVE, 48 | INVERTED_MOTION, 56,
  BALL_MEDIUM | GIFT_NEGATIVE, 80, 56,
  BALL_SMALL | GIFT_NEGATIVE, 112, 26 | INVERTED_MOTION,
};

const unsigned char PROGMEM LTP_C4[] = {
  3 | BACKGROUND_EMPTY_MOON | BACKGROUND_NOISE,
  NO_LUCKY_GIFTS,
  32,
  BALL_SMALL | GIFT_LIFE, 32, 32,
  BALL_MEDIUM | GIFT_SHIELD, 64 , 32,
  BALL_BIG | GIFT_DOUBLE_RAY, 96 , 32,
};

const unsigned char PROGMEM LTP_C5[] = {
  4 | BACKGROUND_EMPTY_MOON | BACKGROUND_MIRROR ,
  NO_LUCKY_GIFTS | GIFT_SLOW,
  32 | ROTATING_BALLS,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 44, 32,
  BALL_MEDIUM, 64, 52,
  BALL_MEDIUM | GIFT_DEATH, 64 | INVERTED_MOTION, 12 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_SHIELD, 84 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};




//
// CLASSIC (CLA) LEVELS
//


const unsigned char PROGMEM CLA_A1[] = {
  3 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  GIFT_DOUBLE_RAY,
  64,
  BALL_MEDIUM, 20 | INVERTED_MOTION, 24,
  BALL_MEDIUM, 64, 12,
  BALL_MEDIUM, 108, 24,
};

const unsigned char PROGMEM CLA_A2[] = {
  8 | BACKGROUND_MOUNTAIN,
  GIFT_NEGATIVE,
  112,
  BALL_SMALL | GIFT_SHIELD, 20 | INVERTED_MOTION, 48,
  BALL_SMALL, 30 | INVERTED_MOTION, 48,
  BALL_SMALL | GIFT_NEGATIVE, 40 | INVERTED_MOTION, 48,
  BALL_SMALL, 50 | INVERTED_MOTION, 48,
  BALL_SMALL | GIFT_NEGATIVE, 60 | INVERTED_MOTION, 48,
  BALL_SMALL, 70 | INVERTED_MOTION, 48,
  BALL_SMALL | GIFT_DEATH, 80 | INVERTED_MOTION, 48,
  BALL_SMALL | GIFT_LIFE, 90 | INVERTED_MOTION, 48,
};


const unsigned char PROGMEM CLA_A3[] = {
  1 | BACKGROUND_MOUNTAIN,
  0,
  12 | ROTATING_BALLS,
  BALL_BIG, 104 | INVERTED_MOTION, 20,
};

const unsigned char PROGMEM CLA_A4[] = {
  4 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR | BACKGROUND_NOISE,
  GIFT_NEGATIVE | GIFT_MIRROR,
  64,
  BALL_SMALL, 18 | INVERTED_MOTION, 50,
  BALL_MEDIUM, 32, 22,
  BALL_MEDIUM, 96 | INVERTED_MOTION, 22,
  BALL_SMALL, 110, 50,
};

const unsigned char PROGMEM CLA_A5[] = {
  2 | BACKGROUND_MOUNTAIN,
  NO_LUCKY_GIFTS | GIFT_SLOW,
  64,
  BALL_BIG | GIFT_FAST, 64 | INVERTED_MOTION, 13,
  BALL_BIG | GIFT_FAST, 64, 13,
};


const unsigned char PROGMEM CLA_B1[] = {
  6 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  0,
  64,
  BALL_SMALL, 52 | INVERTED_MOTION, 49,
  BALL_SMALL, 53 | INVERTED_MOTION, 40,
  BALL_SMALL, 54 | INVERTED_MOTION, 31,
  BALL_SMALL, 74, 31,
  BALL_SMALL, 75, 40,
  BALL_SMALL | GIFT_LIFE, 76, 49,
};

const unsigned char PROGMEM CLA_B2[] = {
  2 | BACKGROUND_CITY,
  GIFT_NEGATIVE,
  100 | ROTATING_BALLS,
  BALL_BIG, 52 , 32,
  BALL_BIG | GIFT_SHIELD, 76 | INVERTED_MOTION, 32,
};

const unsigned char PROGMEM CLA_B3[] = {
  3 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  64,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 31, 20,
  BALL_MEDIUM | GIFT_SLOW, 31, 20,
  BALL_MEDIUM, 31, 20,
};

const unsigned char PROGMEM CLA_B4[] = {
  2 | BACKGROUND_CITY | BACKGROUND_NOISE,
  GIFT_FAST,
  64,
  BALL_MEDIUM | GIFT_NEGATIVE, 30, 14,
  BALL_MEDIUM | GIFT_NEGATIVE, 98 | INVERTED_MOTION, 14 ,
};

const unsigned char PROGMEM CLA_B5[] = {
  3 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  GIFT_SLOW,
  64,
  BALL_BIG | GIFT_LIFE, 24 | INVERTED_MOTION, 24,
  BALL_MEDIUM, 64 | INVERTED_MOTION, 14,
  BALL_BIG | GIFT_DOUBLE_RAY, 104, 24,
};


const unsigned char PROGMEM CLA_C1[] = {
  6 | BACKGROUND_MOON | BACKGROUND_NOISE,
  GIFT_MIRROR,
  64 | ROTATING_BALLS,
  BALL_SMALL, 22, 8,
  BALL_SMALL | GIFT_LIFE, 30, 16,
  BALL_SMALL, 38, 24,
  BALL_SMALL, 90 | INVERTED_MOTION, 24,
  BALL_SMALL | GIFT_DEATH, 98 | INVERTED_MOTION, 16,
  BALL_SMALL, 106 | INVERTED_MOTION, 8,
};

const unsigned char PROGMEM CLA_C2[] = {
  2 | BACKGROUND_EMPTY_MOON | BACKGROUND_MIRROR,
  GIFT_FAST | GIFT_NEGATIVE,
  64,
  BALL_BIG | GIFT_DOUBLE_RAY, 24 | INVERTED_MOTION, 32,
  BALL_BIG | GIFT_DOUBLE_RAY, 104, 32,
};

const unsigned char PROGMEM CLA_C3[] = {
  4 | BACKGROUND_EMPTY_MOON,
  0,
  64,
  BALL_MEDIUM, 64, 24,
  BALL_MEDIUM, 64 | INVERTED_MOTION, 24,
  BALL_MEDIUM, 64, 24 | INVERTED_MOTION,
  BALL_MEDIUM, 64 | INVERTED_MOTION, 24 | INVERTED_MOTION,
};

const unsigned char PROGMEM CLA_C4[] = {
  6 | BACKGROUND_MOON,
  GIFT_NEGATIVE,
  64,
  BALL_BIG | GIFT_LIFE, 45 | INVERTED_MOTION, 52,
  BALL_SMALL, 51 | INVERTED_MOTION, 37,
  BALL_SMALL, 59 | INVERTED_MOTION, 29,
  BALL_SMALL, 69, 29,
  BALL_SMALL, 75, 37,
  BALL_BIG | GIFT_SLOW, 83, 52,
};

const unsigned char PROGMEM CLA_C5[] = {
  4 | BACKGROUND_MOON | BACKGROUND_NOISE,
  0,
  64 | ROTATING_BALLS,
  BALL_BIG | GIFT_SHIELD, 26 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM, 26, 32 | INVERTED_MOTION,
  BALL_BIG, 102, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 102 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};


//
// DIFFICULT (DIFF) LEVELS
//

const unsigned char PROGMEM DIFF_A1[] = {
  10 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS,
  96 | ROTATING_BALLS,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,
  BALL_SMALL, 38, 14,                                         
};


const unsigned char PROGMEM DIFF_A2[] = {
  3 | BACKGROUND_MOUNTAIN,
  NO_LUCKY_GIFTS,
  58,
  BALL_SMALL | GIFT_SHIELD, 48, 25,
  BALL_BIG | GIFT_NEGATIVE, 28, 20,
  BALL_BIG, 88 | INVERTED_MOTION , 20,
};

const unsigned char PROGMEM DIFF_A3[] = {
  2 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR | BACKGROUND_NOISE,
  NO_LUCKY_GIFTS,
  64,
  BALL_BIG | GIFT_SHIELD, 64 | INVERTED_MOTION, 13,
  BALL_BIG | GIFT_LIFE, 64, 13,
};

const unsigned char PROGMEM DIFF_A4[] = {
  1 | BACKGROUND_MOUNTAIN | BACKGROUND_MIRROR,
  GIFT_MIRROR,
  12 | ROTATING_BALLS,
  BALL_BIG, 100 | INVERTED_MOTION, 22,
};

const unsigned char PROGMEM DIFF_A5[] = {
  10 | BACKGROUND_MOUNTAIN,
  GIFT_NEGATIVE,
  64,
  BALL_SMALL, 52 | INVERTED_MOTION, 49,
  BALL_SMALL, 53 | INVERTED_MOTION, 40,
  BALL_SMALL, 54 | INVERTED_MOTION, 31,
  BALL_SMALL, 55 | INVERTED_MOTION, 22,
  BALL_SMALL, 56 | INVERTED_MOTION, 13,      
  BALL_SMALL, 72, 13,
  BALL_SMALL, 73, 22,
  BALL_SMALL, 74, 31,
  BALL_SMALL, 75, 40,
  BALL_SMALL | GIFT_SHIELD, 76, 49,        
};


const unsigned char PROGMEM DIFF_B1[] = {
  3 | BACKGROUND_CITY,
  GIFT_DOUBLE_RAY,
  64,
  BALL_BIG, 24 | INVERTED_MOTION, 24,
  BALL_BIG | GIFT_SHIELD, 64, 14,
  BALL_BIG, 104, 24,
};

const unsigned char PROGMEM DIFF_B2[] = {
  4 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  GIFT_NEGATIVE,
  64 | ROTATING_BALLS,
  BALL_MEDIUM, 30, 14,  
  BALL_MEDIUM | GIFT_SLOW, 34, 32,
  BALL_MEDIUM, 98 | INVERTED_MOTION, 14,
  BALL_MEDIUM, 94 | INVERTED_MOTION, 32,  
};

const unsigned char PROGMEM DIFF_B3[] = {
  4 | BACKGROUND_CITY | BACKGROUND_NOISE,
  0,
  64,
  BALL_BIG, 26 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM, 26, 32 | INVERTED_MOTION,
  BALL_BIG, 102, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 102 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};

const unsigned char PROGMEM DIFF_B4[] = {
  10 | BACKGROUND_CITY | BACKGROUND_MIRROR,
  NO_LUCKY_GIFTS | GIFT_MIRROR,
  64,
  BALL_SMALL | GIFT_SHIELD, 21, 52,
  BALL_SMALL | GIFT_MIRROR, 21, 52,
  BALL_SMALL | GIFT_MIRROR, 21, 52,
  BALL_SMALL | GIFT_MIRROR, 21, 52,
  BALL_SMALL | GIFT_MIRROR, 21, 52,
  BALL_SMALL | GIFT_LIFE, 107 | INVERTED_MOTION, 52,
  BALL_SMALL | GIFT_MIRROR, 107 | INVERTED_MOTION, 52,
  BALL_SMALL | GIFT_MIRROR, 107 | INVERTED_MOTION, 52,
  BALL_SMALL | GIFT_MIRROR, 107 | INVERTED_MOTION, 52,
  BALL_SMALL | GIFT_MIRROR, 107 | INVERTED_MOTION, 52,      
};

const unsigned char PROGMEM DIFF_B5[] = {
  4 | BACKGROUND_CITY,
  NO_LUCKY_GIFTS | GIFT_DOUBLE_RAY,
  64,
  BALL_BIG | GIFT_SHIELD, 16 | INVERTED_MOTION, 24,
  BALL_BIG | GIFT_SHIELD, 48 | INVERTED_MOTION, 24,  
  BALL_BIG | GIFT_SHIELD, 80, 24,
  BALL_BIG | GIFT_SHIELD, 112, 24,
};




const unsigned char PROGMEM DIFF_C1[] = {
  13 | BACKGROUND_MOON,
  0,
  64 | ROTATING_BALLS,
  BALL_SMALL | GIFT_LIFE, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_NEGATIVE, 80 | INVERTED_MOTION, 26 | INVERTED_MOTION,
  BALL_SMALL | GIFT_NEGATIVE, 96 | INVERTED_MOTION, 20 | INVERTED_MOTION,
  BALL_SMALL | GIFT_NEGATIVE, 112 | INVERTED_MOTION, 14 | INVERTED_MOTION,    
  BALL_SMALL | GIFT_NEGATIVE, 80, 40 | INVERTED_MOTION,
  BALL_SMALL | GIFT_NEGATIVE, 96, 46 | INVERTED_MOTION,
  BALL_SMALL | GIFT_NEGATIVE, 112, 52 | INVERTED_MOTION, 
  BALL_SMALL | GIFT_NEGATIVE, 48 | INVERTED_MOTION, 26,
  BALL_SMALL | GIFT_NEGATIVE, 32 | INVERTED_MOTION, 20,
  BALL_SMALL | GIFT_NEGATIVE, 16 | INVERTED_MOTION, 14,  
  BALL_SMALL | GIFT_NEGATIVE, 48, 40,
  BALL_SMALL | GIFT_NEGATIVE, 32, 46,
  BALL_SMALL | GIFT_NEGATIVE, 16, 52,    
};

const unsigned char PROGMEM DIFF_C2[] = {
  4 | BACKGROUND_MOON | BACKGROUND_MIRROR,
  GIFT_FAST | GIFT_NEGATIVE | GIFT_MIRROR,
  64 | ROTATING_BALLS,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 32 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_MIRROR, 32, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_MIRROR, 96, 32 | INVERTED_MOTION,
  BALL_MEDIUM, 96 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};

const unsigned char PROGMEM DIFF_C3[] = {
  10 | BACKGROUND_MOON | BACKGROUND_NOISE | BACKGROUND_MIRROR,
  0,
  64 | ROTATING_BALLS,
  BALL_BIG | GIFT_DOUBLE_RAY, 32 | INVERTED_MOTION, 22,
  BALL_BIG | GIFT_SHIELD, 96, 22,
  BALL_SMALL | GIFT_LIFE, 56 | INVERTED_MOTION, 28 | INVERTED_MOTION,
  BALL_SMALL , 48 | INVERTED_MOTION, 38 | INVERTED_MOTION,
  BALL_SMALL , 48 | INVERTED_MOTION, 48 | INVERTED_MOTION,
  BALL_SMALL , 48 | INVERTED_MOTION, 58 | INVERTED_MOTION,
  BALL_SMALL | GIFT_SLOW, 70, 28 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DEATH, 80, 38 | INVERTED_MOTION,
  BALL_SMALL , 80, 48 | INVERTED_MOTION,
  BALL_SMALL , 80, 58 | INVERTED_MOTION,              
};

const unsigned char PROGMEM DIFF_C4[] = {
  3 | BACKGROUND_MOON,
  NO_LUCKY_GIFTS | GIFT_FAST,
  64,
  BALL_BIG | GIFT_DOUBLE_RAY, 24 | INVERTED_MOTION, 40 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_SHIELD, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_BIG | GIFT_DOUBLE_RAY, 104 | INVERTED_MOTION, 24 | INVERTED_MOTION,
 
};

const unsigned char PROGMEM DIFF_C5[] = {
  4 | BACKGROUND_EMPTY_MOON | BACKGROUND_NOISE,
  NO_LUCKY_GIFTS | GIFT_DOUBLE_RAY | GIFT_FAST,
  64,
  BALL_BIG | GIFT_SHIELD, 64, 20,
  BALL_BIG | GIFT_SHIELD, 64 | INVERTED_MOTION, 20,
  BALL_BIG | GIFT_SHIELD, 64, 20 | INVERTED_MOTION,
  BALL_BIG | GIFT_SHIELD, 64 | INVERTED_MOTION, 20 | INVERTED_MOTION,
};



//
// EXTRA (XTRA) LEVELS
//

const unsigned char PROGMEM XTRA_1[] = {
  4 | BACKGROUND_MOON | BACKGROUND_NOISE,
  0,
  64 | ROTATING_BALLS,
  BALL_BIG, 26 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM, 26, 32 | INVERTED_MOTION,
  BALL_BIG, 102, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 102 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};

//
// TODO MORE LEVELS
//

// 45 levels right now

// Test levels
const unsigned char PROGMEM TEST_15_BALLS[] = {
  15 | BACKGROUND_MOUNTAIN,
  0,
  64,
  BALL_BIG | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_BIG | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_BIG | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_BIG | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_BIG | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_MEDIUM | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
  BALL_SMALL | GIFT_DOUBLE_RAY, 64 | INVERTED_MOTION, 32 | INVERTED_MOTION,
};


const unsigned char PROGMEM LTP_1_TEST[] = {
  1 | BACKGROUND_MOUNTAIN,
  GIFT_SHIELD,
  32,
  BALL_MEDIUM | GIFT_DEATH, 38, 14,
};

#ifdef PERFORMANCE_TEST

const unsigned char *LEVELS[] = {
  TEST_15_BALLS,
  LTP_14,
  LTP_15,
};

#else

const unsigned char *LEVELS[] = {

  LTP_A1,
  LTP_A2,
  LTP_A3,
  LTP_A4,
  LTP_A5,
  
  LTP_B1,
  LTP_B2,
  LTP_B3,
  LTP_B4,
  LTP_B5,

  LTP_C1,
  LTP_C2,
  LTP_C3,
  LTP_C4,
  LTP_C5,

  CLA_A1,
  CLA_A2,
  CLA_A3,
  CLA_A4,
  CLA_A5,
  
  CLA_B1,
  CLA_B2,
  CLA_B3,
  CLA_B4,
  CLA_B5,

  CLA_C1,
  CLA_C2,
  CLA_C3,
  CLA_C4,
  CLA_C5, 

  DIFF_A1,
  DIFF_A2,
  DIFF_A3,
  DIFF_A4,
  DIFF_A5,

  DIFF_B1,
  DIFF_B2,
  DIFF_B3,
  DIFF_B4,
  DIFF_B5,

  DIFF_C1,
  DIFF_C2,
  DIFF_C3,
  DIFF_C4,
  DIFF_C5, 

  //XTRA_1,     

  //TODO More levels
  //...
};

#endif

